/*-----------------------------------------------------------------------------------------------//
Version: 0.3
Authors: Sceptre
Creates a custom dialog with defined control types.
Returns array of selected values from every control.

Parameters:
0: Dialog title <STRING>
1: Arrays of controls array <ARRAY>
	Controls array parameters:
	0: Control type <STRING>
	1: Control text/description <STRING,ARRAY>
	2: Control values <ARRAY>
	3: Control default value <BOOL,SCALAR,STRING>
	4: Use default value <BOOL>
	5: Show/Enable condition <CODE>
	6: Modifier function (_this = _currentValue) <CODE>
	7: Custom arguments used in _ctrlCondition and _ctrlModFunction <ANY>

Returns:
Ordered array of selected values from every control, or [] if dialog cancelled.
//-----------------------------------------------------------------------------------------------*/
#include "defineCDS.sqf"

if (!canSuspend) exitWith {systemChat "CDS_fnc_createDialog must be called in scheduled enviroment"};
params [["_titleText","",[""]],["_controls",[],[[]]]];

// Run following code unscheduled
isNil {([_x] apply {call {
	disableSerialization;
	params [["_titleText","",[""]],["_controls",[],[[]]]];

	CDS_OK = false;
	CDS_currentValues = [];
	createDialog "CDS_Dialog";
	CDS_closed = false;

	private _display = findDisplay DISPLAY_IDD;
	private _neededHeight = count _controls * ROW_H + TITLE_H + BUFFER_H;

	// Background
	private _BG = _display displayCtrl BG_IDC;
	_BG ctrlSetPosition [DEFAULT_X,CENTER_Y - _neededHeight / 2,TOTAL_WIDTH,_neededHeight];
	_BG ctrlCommit 0;

	// Title
	private _title = _display displayCtrl TITLE_IDC;
	_title ctrlSetPosition [DEFAULT_X,CENTER_Y - _neededHeight / 2,TOTAL_WIDTH,TITLE_H];
	_title ctrlSetBackgroundColor [
		profilenamespace getvariable ["GUI_BCG_RGB_R",0.1],
		profilenamespace getvariable ["GUI_BCG_RGB_G",0.1],
		profilenamespace getvariable ["GUI_BCG_RGB_B",0.1],
		profilenamespace getvariable ["GUI_BCG_RGB_A",0.8]
	];
	_title ctrlSetText _titleText;
	_title ctrlCommit 0;

	// Cancel button
	private _cancel = _display displayCtrl CANCEL_IDC;
	_cancel ctrlSetPosition [DEFAULT_X,CENTER_Y + _neededHeight / 2 + BUTTON_SEPARATION,BUTTON_W,BUTTON_H];
	_cancel ctrlCommit 0;

	// OK button
	private _OK = _display displayCtrl OK_IDC;
	_OK ctrlSetPosition [DEFAULT_X + TOTAL_WIDTH - BUTTON_W,CENTER_Y + _neededHeight / 2 + BUTTON_SEPARATION,BUTTON_W,BUTTON_H];
	_OK ctrlCommit 0;

	// Create controls
	CDS_controlsInfo = [];
	private _posY = CENTER_Y - _neededHeight / 2 + TITLE_H + BUFFER_H / 2;
	{
		(_controls # _forEachIndex) params [
			["_ctrlType","",[""]],
			["_ctrlText","",["",[]]],
			["_ctrlValues",[],[[]]],
			["_ctrlDefault","",[true,0,""]],
			["_useDefault",true,[true]],
			["_ctrlCondition",{true},[{}]],
			["_ctrlModFunction",{},[{}]],
			["_ctrlArgs",[]]
		];

		// Try to use a cached value if _useDefault is false
		if !(_useDefault) then {
			private _cachedValue = [_titleText,_ctrlText,_ctrlType,_ctrlValues,_ctrlDefault,true] call CDS_fnc_cacheValue;
			if (!isNil "_cachedValue") then {_ctrlDefault = _cachedValue;};
		};

		// Add text
		private _text = _display ctrlCreate ["CDS_CtrlText",TEXT_IDC_START + _forEachIndex];
		_text ctrlSetPosition [TEXT_X,_posY,TEXT_W,TEXT_H];
		_text ctrlSetBackgroundColor [0,0,0,0];
		if (_ctrlText isEqualType []) then {
			_text ctrlSetText ((_ctrlText # 0) + ":");
			_text ctrlSetTooltip (_ctrlText # 1);
		} else {
			_text ctrlSetText (_ctrlText + ":");
		};
		_text ctrlCommit 0;

		// Add control type
		switch (_ctrlType) do {
			case "CHECKBOX" : {
				if !(_ctrlDefault isEqualType true) then {_ctrlDefault = true;};

				private _ctrl = _display ctrlCreate ["RscCheckbox",CTRL_IDC_START + _forEachIndex];
				_ctrl ctrlSetPosition [CTRL_X,_posY + CHECKBOX_Y,CHECKBOX_W,CHECKBOX_H];
				_ctrl ctrlSetBackgroundColor [0,0,0,0];
				_ctrl cbSetChecked _ctrlDefault;
				_ctrl ctrlCommit 0;
			};
			case "EDITBOX" : {
				if !(_ctrlDefault isEqualType "") then {_ctrlDefault = "";};

				private _ctrl = _display ctrlCreate ["RscEdit",CTRL_IDC_START + _forEachIndex];
				_ctrl ctrlSetPosition [CTRL_X,_posY + EDITBOX_Y,EDITBOX_W,EDITBOX_H];
				_ctrl ctrlSetBackgroundColor [0,0,0,0];
				_ctrl ctrlSetText _ctrlDefault;
				_ctrl ctrlCommit 0;
			};
			case "SLIDER" : {
				if (count _ctrlValues != 3 && {_x isEqualType 0} count _ctrlValues != 3) then {_ctrlValues = [0,1,2]};
				if !(_ctrlDefault isEqualType 0) then {_ctrlDefault = _ctrlValues # 0;};

				private _fixedValue = _ctrlDefault toFixed (_ctrlValues # 2);
				private _ctrl = _display ctrlCreate ["RscXSliderH",CTRL_IDC_START + _forEachIndex];
				_ctrl ctrlSetPosition [CTRL_X,_posY + SLIDER_Y,SLIDER_W,SLIDER_H];
				_ctrl ctrlSetBackgroundColor [0,0,0,0];
				_ctrl sliderSetRange [_ctrlValues # 0,_ctrlValues # 1];
				_ctrl sliderSetPosition _ctrlDefault;
				_ctrl ctrlSetTooltip _fixedValue;
				_ctrl ctrlCommit 0;
				private _ctrlAux = _display ctrlCreate ["RscEdit",CTRL_AUX_IDC_START + _forEachIndex];
				_ctrlAux ctrlSetPosition [SLIDER_AUX_X,_posY + SLIDER_AUX_Y,SLIDER_AUX_W,SLIDER_AUX_H];
				_ctrlAux ctrlSetBackgroundColor [0,0,0,0];
				_ctrlAux ctrlSetText _fixedValue;
				_ctrlAux ctrlCommit 0;
			};
			case "COMBOBOX" : {
				if !(_ctrlDefault isEqualType 0) then {_ctrlDefault = 0;};

				private _ctrl = _display ctrlCreate ["RscCombo",CTRL_IDC_START + _forEachIndex];
				_ctrl ctrlSetPosition [CTRL_X,_posY + COMBOBOX_Y,COMBOBOX_W,COMBOBOX_H];
				_ctrl ctrlSetBackgroundColor [0,0,0,1];
				{
					_ctrl lbAdd _x;
					false
				} count _ctrlValues;
				_ctrl lbSetCurSel _ctrlDefault;
				_ctrl ctrlCommit 0;
			};
			default {systemChat format ["Control %1 : Invalid type",_forEachIndex];};
		};

		_posY = _posY + ROW_H;
		CDS_controlsInfo pushBack [
			CTRL_IDC_START + _forEachIndex,
			TEXT_IDC_START + _forEachIndex,
			_ctrlType,
			_ctrlText,
			_ctrlValues,
			_ctrlDefault,
			_ctrlCondition,
			_ctrlModFunction,
			CTRL_AUX_IDC_START + _forEachIndex,
			str _ctrlDefault,
			_ctrlArgs,
			_titleText
		];
	} forEach _controls;

	// Update dialog controls and values each frame
	CDS_EFID = addMissionEventHandler ["EachFrame",{
		disableSerialization;

		if (!dialog) exitWith {
			if (CDS_OK) then {
				removeMissionEventHandler ["EachFrame",CDS_EFID];
				CDS_OK = false;
				{
					CDS_currentValues pushBack _x # 5;
					[_x # 11,_x # 3,_x # 2,_x # 4,_x # 5,false] call CDS_fnc_cacheValue;
					false
				} count CDS_controlsInfo;
				CDS_closed = true;
			} else {
				removeMissionEventHandler ["EachFrame",CDS_EFID];
				CDS_closed = true;
			};
		};

		private _display = findDisplay DISPLAY_IDD;

		{
			private _ctrl = _display displayCtrl (_x # 0);
			private _ctrlType = _x # 2;

			// Collect current values and update any tooltips
			private _oldValue = _x # 5;
			private _currentValue = switch (_ctrlType) do {
				case "CHECKBOX" : {cbChecked _ctrl};
				case "EDITBOX" : {ctrlText _ctrl};
				case "SLIDER" : {
					(_x # 4) params ["_min","_max","_decimals"];
					private _ctrlValue = sliderPosition _ctrl toFixed _decimals;
					private _ctrlAux = _display displayCtrl (_x # 8);
					private _ctrlAuxValue = abs (_min max (parseNumber ctrlText _ctrlAux) min _max) toFixed _decimals;
					if (_x # 9 != _ctrlAuxValue) then {
						_x set [9,_ctrlAuxValue];
						_ctrl sliderSetPosition parseNumber _ctrlAuxValue;
						_ctrl ctrlSetTooltip _ctrlAuxValue;
						_ctrlAuxValue
					} else {
						_x set [9,_ctrlValue];
						_ctrlAux ctrlSetText _ctrlValue;
						_ctrl ctrlSetTooltip _ctrlValue;
						parseNumber _ctrlValue
					};
				};
				case "COMBOBOX" : {lbCurSel _ctrl};
				default {};
			};

			_x set [5,_currentValue];

			// Execute mod function if value changed
			if !(_oldValue isEqualTo _currentValue) then {
				[_currentValue,_x # 10] call (_x # 7);
			};

			// Toggle control
			private _enableBool = [currentValue,_x # 10] call (_x # 6);
			if (!_enableBool && ctrlEnabled _ctrl) then {
				_ctrl ctrlEnable false;
				_display displayCtrl (_x # 1) ctrlSetTextColor [1,1,1,0.4];
				if (_ctrlType == "SLIDER") then {ctrlEnable [_x # 8,false];};
			};
			if (_enableBool && !ctrlEnabled _ctrl) then {
				_ctrl ctrlEnable true;
				_display displayCtrl (_x # 1) ctrlSetTextColor [1,1,1,1];
				if (_ctrlType == "SLIDER") then {ctrlEnable [_x # 8,true];};
			};

			false
		} count CDS_controlsInfo;
	}];
}}) # 0;};

waitUntil {CDS_closed};
CDS_currentValues
